"""
Version, easily accessible in other files

Copyright (c) 2017 - Eindhoven University of Technology, The Netherlands

This software is made available under the terms of the MIT License.
"""

# TODO: should each tool have its own version?

version_info = (2018, 1, 'dev0')
__version__ = ".".join(map(str, version_info))

package_name = 'nbtoolbelt'
